@@ -17,9 +17,11 @@
 */
 #include <config.h>
 
-#include <inttypes.h>
 #include <glib.h>
+
 #include <common/lz_common.h>
+
+#include "glib-compat.h"
 #include "spice-bitmap-utils.h"
 #include "red-common.h"
 #include "red-qxl.h"
@@ -32,7 +34,7 @@
  * This value should be big enough for all requests but limited
  * to 32 bits. Even better if it fits on 31 bits to detect integer overflows.
  */
-#define MAX_DATA_CHUNK 0x7ffffffflu
+#define MAX_DATA_CHUNK 0x7fffffffLU
 
 verify(MAX_DATA_CHUNK <= G_MAXINT32);
 
@@ -75,6 +77,22 @@ static void hexdump_qxl(RedMemSlotInfo *slots, int group_id,
 }
 #endif
 
+template <typename T>
+inline RedQXLResource<T>::~RedQXLResource()
+{
+    if (qxl) {
+        red_qxl_release_resource(qxl, release_info_ext);
+    }
+}
+
+template <typename T>
+inline void RedQXLResource<T>::set_resource(QXLInstance *qxl_instance, QXLReleaseInfo *info, uint32_t group_id)
+{
+    qxl = qxl_instance;
+    release_info_ext.info = info;
+    release_info_ext.group_id = group_id;
+}
+
 static inline uint32_t color_16_to_32(uint32_t color)
 {
     uint32_t ret;
@@ -98,7 +116,7 @@ static uint8_t *red_linearize_chunk(RedDataChunk *head, size_t size, bool *free_
         return head->data;
     }
 
-    ptr = data = (uint8_t*) g_malloc(size);
+    ptr = data = static_cast<uint8_t *>(g_malloc(size));
     *free_chunk = true;
     for (chunk = head; chunk != nullptr && size > 0; chunk = chunk->next_chunk) {
         copy = MIN(chunk->data_size, size);
@@ -139,7 +157,8 @@ static size_t red_get_data_chunks_ptr(RedMemSlotInfo *slots, int group_id,
         }
 
         memslot_id = memslot_get_id(slots, next_chunk);
-        qxl = (QXLDataChunk *)memslot_get_virt(slots, next_chunk, sizeof(*qxl), group_id);
+        qxl = static_cast<QXLDataChunk *>(
+            memslot_get_virt(slots, next_chunk, sizeof(*qxl), group_id));
         if (qxl == nullptr) {
             goto error;
         }
@@ -192,7 +211,7 @@ static size_t red_get_data_chunks(RedMemSlotInfo *slots, int group_id,
     QXLDataChunk *qxl;
     int memslot_id = memslot_get_id(slots, addr);
 
-    qxl = (QXLDataChunk *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLDataChunk *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return INVALID_SIZE;
     }
@@ -247,7 +266,7 @@ static SpicePath *red_get_path(RedMemSlotInfo *slots, int group_id,
     int i;
     uint32_t count;
 
-    qxl = (QXLPath *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLPath *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return nullptr;
     }
@@ -263,25 +282,25 @@ static SpicePath *red_get_path(RedMemSlotInfo *slots, int group_id,
     n_segments = 0;
     mem_size = sizeof(*red);
 
-    start = (QXLPathSeg*)data;
-    end = (QXLPathSeg*)(data + size);
+    start = reinterpret_cast<QXLPathSeg *>(data);
+    end = reinterpret_cast<QXLPathSeg *>(data + size);
     while (start+1 < end) {
         n_segments++;
         count = start->count;
-        segment_size = sizeof(SpicePathSeg) + (uint64_t) count * sizeof(SpicePointFix);
+        segment_size = sizeof(SpicePathSeg) + uint64_t{count} * sizeof(SpicePointFix);
         mem_size += sizeof(SpicePathSeg *) + SPICE_ALIGN(segment_size, 4);
         /* avoid going backward with 32 bit architectures */
         spice_assert((uint64_t) count * sizeof(QXLPointFix)
                      <= (char*) end - (char*) &start->points[0]);
-        start = (QXLPathSeg*)(&start->points[count]);
+        start = reinterpret_cast<QXLPathSeg *>(&start->points[count]);
     }
 
-    red = (SpicePath*) g_malloc(mem_size);
+    red = static_cast<SpicePath *>(g_malloc(mem_size));
     red->num_segments = n_segments;
 
-    start = (QXLPathSeg*)data;
-    end = (QXLPathSeg*)(data + size);
-    seg = (SpicePathSeg*)&red->segments[n_segments];
+    start = reinterpret_cast<QXLPathSeg *>(data);
+    end = reinterpret_cast<QXLPathSeg *>(data + size);
+    seg = reinterpret_cast<SpicePathSeg *>(&red->segments[n_segments]);
     n_segments = 0;
     mem_size2 = sizeof(*red);
     while (start+1 < end && n_segments < red->num_segments) {
@@ -291,7 +310,7 @@ static SpicePath *red_get_path(RedMemSlotInfo *slots, int group_id,
         /* Protect against overflow in size calculations before
            writing to memory */
         /* Verify that we didn't overflow due to guest changing data */
-        mem_size2 += sizeof(SpicePathSeg) + (uint64_t) count * sizeof(SpicePointFix);
+        mem_size2 += sizeof(SpicePathSeg) + uint64_t{count} * sizeof(SpicePointFix);
         spice_assert(mem_size2 <= mem_size);
 
         seg->flags = start->flags;
@@ -300,8 +319,8 @@ static SpicePath *red_get_path(RedMemSlotInfo *slots, int group_id,
             seg->points[i].x = start->points[i].x;
             seg->points[i].y = start->points[i].y;
         }
-        start = (QXLPathSeg*)(&start->points[i]);
-        seg = (SpicePathSeg*)(&seg->points[i]);
+        start = reinterpret_cast<QXLPathSeg *>(&start->points[i]);
+        seg = reinterpret_cast<SpicePathSeg *>(&seg->points[i]);
     }
     /* Ensure guest didn't tamper with segment count */
     spice_assert(n_segments == red->num_segments);
@@ -325,7 +344,7 @@ static SpiceClipRects *red_get_clip_rects(RedMemSlotInfo *slots, int group_id,
     int i;
     uint32_t num_rects;
 
-    qxl = (QXLClipRects *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLClipRects *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return nullptr;
     }
@@ -345,10 +364,10 @@ static SpiceClipRects *red_get_clip_rects(RedMemSlotInfo *slots, int group_id,
      */
     spice_assert((uint64_t) num_rects * sizeof(QXLRect) == size);
     SPICE_VERIFY(sizeof(SpiceRect) == sizeof(QXLRect));
-    red = (SpiceClipRects*) g_malloc(sizeof(*red) + num_rects * sizeof(SpiceRect));
+    red = static_cast<SpiceClipRects *>(g_malloc(sizeof(*red) + num_rects * sizeof(SpiceRect)));
     red->num_rects = num_rects;
 
-    start = (QXLRect*)data;
+    start = reinterpret_cast<QXLRect *>(data);
     for (i = 0; i < red->num_rects; i++) {
         red_get_rect_ptr(red->rects + i, start++);
     }
@@ -372,7 +391,7 @@ static SpiceChunks *red_get_image_data_flat(RedMemSlotInfo *slots, int group_id,
 
     data = spice_chunks_new(1);
     data->data_size      = size;
-    data->chunk[0].data  = (uint8_t*) bitmap_virt;
+    data->chunk[0].data = static_cast<uint8_t *>(bitmap_virt);
     data->chunk[0].len   = size;
     return data;
 }
@@ -433,7 +452,7 @@ static bool bitmap_consistent(SpiceBitmap *bitmap)
 
     bpp = MAP_BITMAP_FMT_TO_BITS_PER_PIXEL[bitmap->format];
 
-    if (bitmap->stride < (((uint64_t) bitmap->x * bpp + 7u) / 8u)) {
+    if (bitmap->stride < ((uint64_t{bitmap->x} * bpp + 7U) / 8U)) {
         spice_warning("image stride too small for width: %d < ((%d * %d + 7) / 8) (%s=%d)",
                     bitmap->stride, bitmap->x, bpp,
                     bitmap_format_to_string(bitmap->format),
@@ -458,7 +477,7 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
         return nullptr;
     }
 
-    qxl = (QXLImage *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLImage *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return nullptr;
     }
@@ -501,8 +520,7 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
         if (palette) {
             QXLPalette *qp;
             int i, num_ents;
-            qp = (QXLPalette *)memslot_get_virt(slots, palette,
-                                                sizeof(*qp), group_id);
+            qp = static_cast<QXLPalette *>(memslot_get_virt(slots, palette, sizeof(*qp), group_id));
             if (qp == nullptr) {
                 goto error;
             }
@@ -512,7 +530,8 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
                                        num_ents * sizeof(qp->ents[0]), group_id)) {
                 goto error;
             }
-            rp = (SpicePalette*) g_malloc(num_ents * sizeof(rp->ents[0]) + sizeof(*rp));
+            rp =
+                static_cast<SpicePalette *>(g_malloc(num_ents * sizeof(rp->ents[0]) + sizeof(*rp)));
             rp->unique   = qp->unique;
             rp->num_ents = num_ents;
             if (flags & QXL_COMMAND_FLAG_COMPAT_16BPP) {
@@ -527,7 +546,7 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
             red->u.bitmap.palette = rp;
             red->u.bitmap.palette_id = rp->unique;
         }
-        bitmap_size = (uint64_t) red->u.bitmap.y * red->u.bitmap.stride;
+        bitmap_size = uint64_t{red->u.bitmap.y} * red->u.bitmap.stride;
         if (bitmap_size > MAX_DATA_CHUNK) {
             goto error;
         }
@@ -535,6 +554,9 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
             red->u.bitmap.data = red_get_image_data_flat(slots, group_id,
                                                          qxl->bitmap.data,
                                                          bitmap_size);
+            if (red->u.bitmap.data == nullptr) {
+                goto error;
+            }
         } else {
             size = red_get_data_chunks(slots, group_id,
                                        &chunks, qxl->bitmap.data);
@@ -555,9 +577,8 @@ static SpiceImage *red_get_image(RedMemSlotInfo *slots, int group_id,
         break;
     case SPICE_IMAGE_TYPE_QUIC:
         red->u.quic.data_size = qxl->quic.data_size;
-        size = red_get_data_chunks_ptr(slots, group_id,
-                                       memslot_get_id(slots, addr),
-                                       &chunks, (QXLDataChunk *)qxl->quic.data);
+        size = red_get_data_chunks_ptr(slots, group_id, memslot_get_id(slots, addr), &chunks,
+                                       reinterpret_cast<QXLDataChunk *>(qxl->quic.data));
         if (size == INVALID_SIZE || size != red->u.quic.data_size) {
             red_put_data_chunks(&chunks);
             goto error;
@@ -767,7 +788,8 @@ static bool get_transform(RedMemSlotInfo *slots,
     if (qxl_transform == 0)
         return false;
 
-    t = (uint32_t *)memslot_get_virt(slots, qxl_transform, sizeof(*dst_transform), group_id);
+    t = static_cast<uint32_t *>(
+        memslot_get_virt(slots, qxl_transform, sizeof(*dst_transform), group_id));
 
     if (t == nullptr)
         return false;
@@ -837,11 +859,12 @@ static bool red_get_stroke_ptr(RedMemSlotInfo *slots, int group_id,
         uint8_t *buf;
 
         style_nseg = qxl->attr.style_nseg;
-        red->attr.style = (SPICE_FIXED28_4*) g_malloc_n(style_nseg, sizeof(SPICE_FIXED28_4));
+        red->attr.style =
+            static_cast<SPICE_FIXED28_4 *>(g_malloc_n(style_nseg, sizeof(SPICE_FIXED28_4)));
         red->attr.style_nseg  = style_nseg;
         spice_assert(qxl->attr.style);
-        buf = (uint8_t *)memslot_get_virt(slots, qxl->attr.style,
-                                          style_nseg * sizeof(QXLFIXED), group_id);
+        buf = static_cast<uint8_t *>(
+            memslot_get_virt(slots, qxl->attr.style, style_nseg * sizeof(QXLFIXED), group_id));
         if (buf == nullptr) {
             return false;
         }
@@ -881,7 +904,7 @@ static SpiceString *red_get_string(RedMemSlotInfo *slots, int group_id,
     unsigned int bpp = 0;
     uint16_t qxl_flags, qxl_length;
 
-    qxl = (QXLString *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLString *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return nullptr;
     }
@@ -908,32 +931,32 @@ static SpiceString *red_get_string(RedMemSlotInfo *slots, int group_id,
     }
     spice_assert(bpp != 0);
 
-    start = (QXLRasterGlyph*)data;
-    end = (QXLRasterGlyph*)(data + chunk_size);
+    start = reinterpret_cast<QXLRasterGlyph *>(data);
+    end = reinterpret_cast<QXLRasterGlyph *>(data + chunk_size);
     red_size = sizeof(SpiceString);
     glyphs = 0;
     while (start < end) {
         spice_assert((QXLRasterGlyph*)(&start->data[0]) <= end);
         glyphs++;
-        glyph_size = start->height * ((start->width * bpp + 7u) / 8u);
+        glyph_size = start->height * ((start->width * bpp + 7U) / 8U);
         red_size += sizeof(SpiceRasterGlyph *) + SPICE_ALIGN(sizeof(SpiceRasterGlyph) + glyph_size, 4);
         /* do the test correctly, we know end - start->data[0] cannot
          * overflow, don't use start->data[glyph_size] to test for
          * buffer overflow as this on 32 bit can cause overflow
          * on the pointer arithmetic */
         spice_assert(glyph_size <= (char*) end - (char*) &start->data[0]);
-        start = (QXLRasterGlyph*)(&start->data[glyph_size]);
+        start = reinterpret_cast<QXLRasterGlyph *>(&start->data[glyph_size]);
     }
     spice_assert(start <= end);
     spice_assert(glyphs == qxl_length);
 
-    red = (SpiceString*) g_malloc(red_size);
+    red = static_cast<SpiceString *>(g_malloc(red_size));
     red->length = qxl_length;
     red->flags = qxl_flags;
 
-    start = (QXLRasterGlyph*)data;
-    end = (QXLRasterGlyph*)(data + chunk_size);
-    glyph = (SpiceRasterGlyph *)&red->glyphs[red->length];
+    start = reinterpret_cast<QXLRasterGlyph *>(data);
+    end = reinterpret_cast<QXLRasterGlyph *>(data + chunk_size);
+    glyph = reinterpret_cast<SpiceRasterGlyph *>(&red->glyphs[red->length]);
     for (i = 0; i < red->length; i++) {
         spice_assert((QXLRasterGlyph*)(&start->data[0]) <= end);
         red->glyphs[i] = glyph;
@@ -941,11 +964,11 @@ static SpiceString *red_get_string(RedMemSlotInfo *slots, int group_id,
         glyph->height = start->height;
         red_get_point_ptr(&glyph->render_pos, &start->render_pos);
         red_get_point_ptr(&glyph->glyph_origin, &start->glyph_origin);
-        glyph_size = glyph->height * ((glyph->width * bpp + 7u) / 8u);
+        glyph_size = glyph->height * ((glyph->width * bpp + 7U) / 8U);
         /* see above for similar test */
         spice_assert(glyph_size <= (char*) end - (char*) &start->data[0]);
         memcpy(glyph->data, start->data, glyph_size);
-        start = (QXLRasterGlyph*)(&start->data[glyph_size]);
+        start = reinterpret_cast<QXLRasterGlyph *>(&start->data[glyph_size]);
         glyph = SPICE_ALIGNED_CAST(SpiceRasterGlyph*,
             (((uint8_t *)glyph) +
              SPICE_ALIGN(sizeof(SpiceRasterGlyph) + glyph_size, 4)));
@@ -1017,13 +1040,11 @@ static bool red_get_native_drawable(QXLInstance *qxl_instance, RedMemSlotInfo *s
     QXLDrawable *qxl;
     int i;
 
-    qxl = (QXLDrawable *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLDrawable *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info     = &qxl->release_info;
-    red->release_info_ext.group_id = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
 
     red_get_rect_ptr(&red->bbox, &qxl->bbox);
     red_get_clip_ptr(slots, group_id, &red->clip, &qxl->clip);
@@ -1097,13 +1118,11 @@ static bool red_get_compat_drawable(QXLInstance *qxl_instance, RedMemSlotInfo *s
 {
     QXLCompatDrawable *qxl;
 
-    qxl = (QXLCompatDrawable *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLCompatDrawable *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info     = &qxl->release_info;
-    red->release_info_ext.group_id = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
 
     red_get_rect_ptr(&red->bbox, &qxl->bbox);
     red_get_clip_ptr(slots, group_id, &red->clip, &qxl->clip);
@@ -1188,101 +1207,78 @@ static bool red_get_drawable(QXLInstance *qxl, RedMemSlotInfo *slots, int group_
     return ret;
 }
 
-static void red_put_drawable(RedDrawable *red)
+RedDrawable::~RedDrawable()
 {
-    red_put_clip(&red->clip);
-    if (red->self_bitmap_image) {
-        red_put_image(red->self_bitmap_image);
+    red_put_clip(&clip);
+    if (self_bitmap_image) {
+        red_put_image(self_bitmap_image);
     }
-    switch (red->type) {
+    switch (type) {
     case QXL_DRAW_ALPHA_BLEND:
-        red_put_alpha_blend(&red->u.alpha_blend);
+        red_put_alpha_blend(&u.alpha_blend);
         break;
     case QXL_DRAW_BLACKNESS:
-        red_put_blackness(&red->u.blackness);
+        red_put_blackness(&u.blackness);
         break;
     case QXL_DRAW_BLEND:
-        red_put_blend(&red->u.blend);
+        red_put_blend(&u.blend);
         break;
     case QXL_DRAW_COPY:
-        red_put_copy(&red->u.copy);
+        red_put_copy(&u.copy);
         break;
     case QXL_DRAW_FILL:
-        red_put_fill(&red->u.fill);
+        red_put_fill(&u.fill);
         break;
     case QXL_DRAW_OPAQUE:
-        red_put_opaque(&red->u.opaque);
+        red_put_opaque(&u.opaque);
         break;
     case QXL_DRAW_INVERS:
-        red_put_invers(&red->u.invers);
+        red_put_invers(&u.invers);
         break;
     case QXL_DRAW_ROP3:
-        red_put_rop3(&red->u.rop3);
+        red_put_rop3(&u.rop3);
         break;
     case QXL_DRAW_COMPOSITE:
-        red_put_composite(&red->u.composite);
+        red_put_composite(&u.composite);
         break;
     case QXL_DRAW_STROKE:
-        red_put_stroke(&red->u.stroke);
+        red_put_stroke(&u.stroke);
         break;
     case QXL_DRAW_TEXT:
-        red_put_text_ptr(&red->u.text);
+        red_put_text_ptr(&u.text);
         break;
     case QXL_DRAW_TRANSPARENT:
-        red_put_transparent(&red->u.transparent);
+        red_put_transparent(&u.transparent);
         break;
     case QXL_DRAW_WHITENESS:
-        red_put_whiteness(&red->u.whiteness);
+        red_put_whiteness(&u.whiteness);
         break;
     }
-    if (red->qxl != nullptr) {
-        red_qxl_release_resource(red->qxl, red->release_info_ext);
-    }
 }
 
-RedDrawable *red_drawable_new(QXLInstance *qxl, RedMemSlotInfo *slots,
-                              int group_id, QXLPHYSICAL addr,
-                              uint32_t flags)
+red::shared_ptr<RedDrawable>
+red_drawable_new(QXLInstance *qxl, RedMemSlotInfo *slots,
+                 int group_id, QXLPHYSICAL addr, uint32_t flags)
 {
-    auto red = g_new0(RedDrawable, 1);
+    auto red = red::make_shared<RedDrawable>();
 
-    red->refs = 1;
-
-    if (!red_get_drawable(qxl, slots, group_id, red, addr, flags)) {
-       red_drawable_unref(red);
-       return nullptr;
+    if (!red_get_drawable(qxl, slots, group_id, red.get(), addr, flags)) {
+        red.reset();
     }
 
     return red;
 }
 
-RedDrawable *red_drawable_ref(RedDrawable *drawable)
-{
-    drawable->refs++;
-    return drawable;
-}
-
-void red_drawable_unref(RedDrawable *red_drawable)
-{
-    if (--red_drawable->refs) {
-        return;
-    }
-    red_put_drawable(red_drawable);
-    g_free(red_drawable);
-}
-
 static bool red_get_update_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots, int group_id,
                                RedUpdateCmd *red, QXLPHYSICAL addr)
 {
     QXLUpdateCmd *qxl;
 
-    qxl = (QXLUpdateCmd *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLUpdateCmd *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info     = &qxl->release_info;
-    red->release_info_ext.group_id = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
 
     red_get_rect_ptr(&red->area, &qxl->area);
     red->update_id  = qxl->update_id;
@@ -1290,45 +1286,21 @@ static bool red_get_update_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots,
     return true;
 }
 
-static void red_put_update_cmd(RedUpdateCmd *red)
+RedUpdateCmd::~RedUpdateCmd() = default;
+
+red::shared_ptr<const RedUpdateCmd>
+red_update_cmd_new(QXLInstance *qxl, RedMemSlotInfo *slots,
+                   int group_id, QXLPHYSICAL addr)
 {
-    if (red->qxl != nullptr) {
-        red_qxl_release_resource(red->qxl, red->release_info_ext);
-    }
-}
+    auto red = red::make_shared<RedUpdateCmd>();
 
-RedUpdateCmd *red_update_cmd_new(QXLInstance *qxl, RedMemSlotInfo *slots,
-                                 int group_id, QXLPHYSICAL addr)
-{
-    RedUpdateCmd *red;
-
-    red = g_new0(RedUpdateCmd, 1);
-
-    red->refs = 1;
-
-    if (!red_get_update_cmd(qxl, slots, group_id, red, addr)) {
-        red_update_cmd_unref(red);
-        return nullptr;
+    if (!red_get_update_cmd(qxl, slots, group_id, red.get(), addr)) {
+        red.reset();
     }
 
     return red;
 }
 
-RedUpdateCmd *red_update_cmd_ref(RedUpdateCmd *red)
-{
-    red->refs++;
-    return red;
-}
-
-void red_update_cmd_unref(RedUpdateCmd *red)
-{
-    if (--red->refs) {
-        return;
-    }
-    red_put_update_cmd(red);
-    g_free(red);
-}
-
 static bool red_get_message(QXLInstance *qxl_instance, RedMemSlotInfo *slots, int group_id,
                             RedMessage *red, QXLPHYSICAL addr)
 {
@@ -1343,18 +1315,16 @@ static bool red_get_message(QXLInstance *qxl_instance, RedMemSlotInfo *slots, in
      *   luckily this is for debug logging only,
      *   so we can just ignore it by default.
      */
-    qxl = (QXLMessage *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLMessage *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info      = &qxl->release_info;
-    red->release_info_ext.group_id  = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
     red->data                       = qxl->data;
     memslot_id = memslot_get_id(slots, addr+sizeof(*qxl));
     len = memslot_max_size_virt(slots, ((intptr_t) qxl)+sizeof(*qxl), memslot_id, group_id);
     len = MIN(len, 100000);
-    end = (uint8_t *)memchr(qxl->data, 0, len);
+    end = static_cast<uint8_t *>(memchr(qxl->data, 0, len));
     if (end == nullptr) {
         return false;
     }
@@ -1362,45 +1332,21 @@ static bool red_get_message(QXLInstance *qxl_instance, RedMemSlotInfo *slots, in
     return true;
 }
 
-static void red_put_message(RedMessage *red)
+RedMessage::~RedMessage() = default;
+
+red::shared_ptr<const RedMessage>
+red_message_new(QXLInstance *qxl, RedMemSlotInfo *slots,
+                int group_id, QXLPHYSICAL addr)
 {
-    if (red->qxl != nullptr) {
-        red_qxl_release_resource(red->qxl, red->release_info_ext);
-    }
-}
+    auto red = red::make_shared<RedMessage>();
 
-RedMessage *red_message_new(QXLInstance *qxl, RedMemSlotInfo *slots,
-                            int group_id, QXLPHYSICAL addr)
-{
-    RedMessage *red;
-
-    red = g_new0(RedMessage, 1);
-
-    red->refs = 1;
-
-    if (!red_get_message(qxl, slots, group_id, red, addr)) {
-        red_message_unref(red);
-        return nullptr;
+    if (!red_get_message(qxl, slots, group_id, red.get(), addr)) {
+        red.reset();
     }
 
     return red;
 }
 
-RedMessage *red_message_ref(RedMessage *red)
-{
-    red->refs++;
-    return red;
-}
-
-void red_message_unref(RedMessage *red)
-{
-    if (--red->refs) {
-        return;
-    }
-    red_put_message(red);
-    g_free(red);
-}
-
 static unsigned int surface_format_to_bpp(uint32_t format)
 {
     switch (format) {
@@ -1432,14 +1378,14 @@ bool red_validate_surface(uint32_t width, uint32_t height,
     }
 
     /* check stride is larger than required bytes */
-    size = ((uint64_t) width * bpp + 7u) / 8u;
+    size = (uint64_t{width} * bpp + 7U) / 8U;
     /* the uint32_t conversion is here to avoid problems with -2^31 value */
-    if (stride == G_MININT32 || size > (uint32_t) abs(stride)) {
+    if (stride == G_MININT32 || size > uint32_t{abs(stride)}) {
         return false;
     }
 
     /* the multiplication can overflow, also abs(-2^31) may return a negative value */
-    size = (uint64_t) height * abs(stride);
+    size = uint64_t{height} * abs(stride);
     return size <= MAX_DATA_CHUNK;
 }
 
@@ -1450,13 +1396,11 @@ static bool red_get_surface_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots
     QXLSurfaceCmd *qxl;
     uint64_t size;
 
-    qxl = (QXLSurfaceCmd *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLSurfaceCmd *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info      = &qxl->release_info;
-    red->release_info_ext.group_id  = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
 
     red->surface_id = qxl->surface_id;
     red->type       = qxl->type;
@@ -1475,8 +1419,8 @@ static bool red_get_surface_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots
         }
 
         size = red->u.surface_create.height * abs(red->u.surface_create.stride);
-        red->u.surface_create.data =
-            (uint8_t*)memslot_get_virt(slots, qxl->u.surface_create.data, size, group_id);
+        red->u.surface_create.data = static_cast<uint8_t *>(
+            memslot_get_virt(slots, qxl->u.surface_create.data, size, group_id));
         if (red->u.surface_create.data == nullptr) {
             return false;
         }
@@ -1485,45 +1429,21 @@ static bool red_get_surface_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots
     return true;
 }
 
-static void red_put_surface_cmd(RedSurfaceCmd *red)
+RedSurfaceCmd::~RedSurfaceCmd() = default;
+
+red::shared_ptr<const RedSurfaceCmd>
+red_surface_cmd_new(QXLInstance *qxl_instance, RedMemSlotInfo *slots,
+                    int group_id, QXLPHYSICAL addr)
 {
-    if (red->qxl) {
-        red_qxl_release_resource(red->qxl, red->release_info_ext);
-    }
-}
+    auto cmd = red::make_shared<RedSurfaceCmd>();
 
-RedSurfaceCmd *red_surface_cmd_new(QXLInstance *qxl_instance, RedMemSlotInfo *slots,
-                                   int group_id, QXLPHYSICAL addr)
-{
-    RedSurfaceCmd *cmd;
-
-    cmd = g_new0(RedSurfaceCmd, 1);
-
-    cmd->refs = 1;
-
-    if (!red_get_surface_cmd(qxl_instance, slots, group_id, cmd, addr)) {
-        red_surface_cmd_unref(cmd);
-        return nullptr;
+    if (!red_get_surface_cmd(qxl_instance, slots, group_id, cmd.get(), addr)) {
+        cmd.reset();
     }
 
     return cmd;
 }
 
-RedSurfaceCmd *red_surface_cmd_ref(RedSurfaceCmd *cmd)
-{
-    cmd->refs++;
-    return cmd;
-}
-
-void red_surface_cmd_unref(RedSurfaceCmd *cmd)
-{
-    if (--cmd->refs) {
-        return;
-    }
-    red_put_surface_cmd(cmd);
-    g_free(cmd);
-}
-
 static bool red_get_cursor(RedMemSlotInfo *slots, int group_id,
                            SpiceCursor *red, QXLPHYSICAL addr)
 {
@@ -1533,7 +1453,7 @@ static bool red_get_cursor(RedMemSlotInfo *slots, int group_id,
     uint8_t *data;
     bool free_data;
 
-    qxl = (QXLCursor *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLCursor *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
@@ -1559,7 +1479,7 @@ static bool red_get_cursor(RedMemSlotInfo *slots, int group_id,
     if (free_data) {
         red->data = data;
     } else {
-        red->data = (uint8_t*) g_memdup(data, size);
+        red->data = static_cast<uint8_t *>(g_memdup2(data, size));
     }
     // Arrived here we could note that we are not going to use anymore cursor data
     // and we could be tempted to release resource back to QXL. Don't do that!
@@ -1579,13 +1499,11 @@ static bool red_get_cursor_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots,
 {
     QXLCursorCmd *qxl;
 
-    qxl = (QXLCursorCmd *)memslot_get_virt(slots, addr, sizeof(*qxl), group_id);
+    qxl = static_cast<QXLCursorCmd *>(memslot_get_virt(slots, addr, sizeof(*qxl), group_id));
     if (qxl == nullptr) {
         return false;
     }
-    red->qxl = qxl_instance;
-    red->release_info_ext.info      = &qxl->release_info;
-    red->release_info_ext.group_id  = group_id;
+    red->set_resource(qxl_instance, &qxl->release_info, group_id);
 
     red->type = qxl->type;
     switch (red->type) {
@@ -1604,46 +1522,23 @@ static bool red_get_cursor_cmd(QXLInstance *qxl_instance, RedMemSlotInfo *slots,
     return true;
 }
 
-RedCursorCmd *red_cursor_cmd_new(QXLInstance *qxl, RedMemSlotInfo *slots,
-                                 int group_id, QXLPHYSICAL addr)
+red::shared_ptr<const RedCursorCmd>
+red_cursor_cmd_new(QXLInstance *qxl, RedMemSlotInfo *slots, int group_id, QXLPHYSICAL addr)
 {
-    RedCursorCmd *cmd;
+    auto cmd = red::make_shared<RedCursorCmd>();
 
-    cmd = g_new0(RedCursorCmd, 1);
-
-    cmd->refs = 1;
-
-    if (!red_get_cursor_cmd(qxl, slots, group_id, cmd, addr)) {
-        red_cursor_cmd_unref(cmd);
-        return nullptr;
+    if (!red_get_cursor_cmd(qxl, slots, group_id, cmd.get(), addr)) {
+        cmd.reset();
     }
 
     return cmd;
 }
 
-static void red_put_cursor_cmd(RedCursorCmd *red)
+RedCursorCmd::~RedCursorCmd()
 {
-    switch (red->type) {
+    switch (type) {
     case QXL_CURSOR_SET:
-        red_put_cursor(&red->u.set.shape);
+        red_put_cursor(&u.set.shape);
         break;
     }
-    if (red->qxl) {
-        red_qxl_release_resource(red->qxl, red->release_info_ext);
-    }
-}
-
-RedCursorCmd *red_cursor_cmd_ref(RedCursorCmd *red)
-{
-    red->refs++;
-    return red;
-}
-
-void red_cursor_cmd_unref(RedCursorCmd *red)
-{
-    if (--red->refs) {
-        return;
-    }
-    red_put_cursor_cmd(red);
-    g_free(red);
 }
