@@ -20,9 +20,11 @@
 
 #include <openssl/ssl.h>
 
-#include "spice.h"
+#include "spice-wrapped.h"
 #include "red-common.h"
 
+SPICE_BEGIN_DECLS
+
 typedef void (*AsyncReadDone)(void *opaque);
 typedef void (*AsyncReadError)(void *opaque, int err);
 
@@ -103,4 +105,6 @@ typedef enum {
 typedef void (*RedSaslResult)(void *opaque, RedSaslError err);
 bool red_sasl_start_auth(RedStream *stream, RedSaslResult result_cb, void *opaque);
 
+SPICE_END_DECLS
+
 #endif /* RED_STREAM_H_ */
