@@ -20,15 +20,16 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
-#include <stdio.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <string.h>
-#include <unistd.h>
-#include <pthread.h>
-#include <inttypes.h>
+#include <cerrno>
+#include <cinttypes>
+#include <cstdarg>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+
 #include <glib.h>
+#include <pthread.h>
+#include <unistd.h>
 
 #include <spice/protocol.h>
 #include <spice/qxl_dev.h>
@@ -90,16 +91,13 @@ struct RedWorker {
 
 static gboolean red_process_cursor_cmd(RedWorker *worker, const QXLCommandExt *ext)
 {
-    RedCursorCmd *cursor_cmd;
-
-    cursor_cmd = red_cursor_cmd_new(worker->qxl, &worker->mem_slots,
-                                    ext->group_id, ext->cmd.data);
-    if (cursor_cmd == nullptr) {
+    auto cursor_cmd = red_cursor_cmd_new(worker->qxl, &worker->mem_slots,
+                                         ext->group_id, ext->cmd.data);
+    if (!cursor_cmd) {
         return FALSE;
     }
 
-    worker->cursor_channel->process_cmd(cursor_cmd);
-    red_cursor_cmd_unref(cursor_cmd);
+    worker->cursor_channel->process_cmd(std::move(cursor_cmd));
 
     return TRUE;
 }
@@ -148,15 +146,12 @@ static int red_process_cursor(RedWorker *worker, int *ring_is_empty)
 
 static gboolean red_process_surface_cmd(RedWorker *worker, QXLCommandExt *ext, gboolean loadvm)
 {
-    RedSurfaceCmd *surface_cmd;
-
-    surface_cmd = red_surface_cmd_new(worker->qxl, &worker->mem_slots,
-                                      ext->group_id, ext->cmd.data);
-    if (surface_cmd == nullptr) {
+    auto surface_cmd = red_surface_cmd_new(worker->qxl, &worker->mem_slots,
+                                           ext->group_id, ext->cmd.data);
+    if (!surface_cmd) {
         return false;
     }
-    display_channel_process_surface_cmd(worker->display_channel, surface_cmd, loadvm);
-    red_surface_cmd_unref(surface_cmd);
+    display_channel_process_surface_cmd(worker->display_channel, std::move(surface_cmd), loadvm);
 
     return true;
 }
@@ -197,24 +192,20 @@ static int red_process_display(RedWorker *worker, int *ring_is_empty)
         worker->display_poll_tries = 0;
         switch (ext_cmd.cmd.type) {
         case QXL_CMD_DRAW: {
-            RedDrawable *red_drawable;
-            red_drawable = red_drawable_new(worker->qxl, &worker->mem_slots,
-                                            ext_cmd.group_id, ext_cmd.cmd.data,
-                                            ext_cmd.flags); // returns with 1 ref
+            auto red_drawable = red_drawable_new(worker->qxl, &worker->mem_slots,
+                                                 ext_cmd.group_id, ext_cmd.cmd.data,
+                                                 ext_cmd.flags); // returns with 1 ref
 
-            if (red_drawable != nullptr) {
-                display_channel_process_draw(worker->display_channel, red_drawable,
+            if (red_drawable) {
+                display_channel_process_draw(worker->display_channel, std::move(red_drawable),
                                              worker->process_display_generation);
-                red_drawable_unref(red_drawable);
             }
             break;
         }
         case QXL_CMD_UPDATE: {
-            RedUpdateCmd *update;
-
-            update = red_update_cmd_new(worker->qxl, &worker->mem_slots,
-                                        ext_cmd.group_id, ext_cmd.cmd.data);
-            if (update == nullptr) {
+            auto update = red_update_cmd_new(worker->qxl, &worker->mem_slots,
+                                             ext_cmd.group_id, ext_cmd.cmd.data);
+            if (!update) {
                 break;
             }
             if (!display_channel_validate_surface(worker->display_channel, update->surface_id)) {
@@ -223,21 +214,17 @@ static int red_process_display(RedWorker *worker, int *ring_is_empty)
                 display_channel_draw(worker->display_channel, &update->area, update->surface_id);
                 red_qxl_notify_update(worker->qxl, update->update_id);
             }
-            red_update_cmd_unref(update);
             break;
         }
         case QXL_CMD_MESSAGE: {
-            RedMessage *message;
-
-            message = red_message_new(worker->qxl, &worker->mem_slots,
-                                      ext_cmd.group_id, ext_cmd.cmd.data);
-            if (message == nullptr) {
+            auto message = red_message_new(worker->qxl, &worker->mem_slots,
+                                           ext_cmd.group_id, ext_cmd.cmd.data);
+            if (!message) {
                 break;
             }
 #ifdef DEBUG
             spice_warning("MESSAGE: %.*s", message->len, message->data);
 #endif
-            red_message_unref(message);
             break;
         }
         case QXL_CMD_SURFACE:
@@ -265,7 +252,7 @@ static bool red_process_is_blocked(RedWorker *worker)
            worker->display_channel->max_pipe_size() > MAX_PIPE_SIZE;
 }
 
-typedef int (*red_process_t)(RedWorker *worker, int *ring_is_empty);
+using red_process_t = int (*)(RedWorker *worker, int *ring_is_empty);
 static void flush_commands(RedWorker *worker, RedChannel *red_channel,
                            red_process_t process)
 {
@@ -329,10 +316,9 @@ static void flush_all_qxl_commands(RedWorker *worker)
     flush_cursor_commands(worker);
 }
 
-static void handle_dev_update_async(void *opaque, void *payload)
+static void
+handle_dev_update_async(RedWorker* worker, RedWorkerMessageUpdateAsync* msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageUpdateAsync*) payload;
     QXLRect *qxl_dirty_rects = nullptr;
     uint32_t num_dirty_rects = 0;
 
@@ -350,10 +336,9 @@ static void handle_dev_update_async(void *opaque, void *payload)
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_update(void *opaque, void *payload)
+static void
+handle_dev_update(RedWorker *worker, RedWorkerMessageUpdate *msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageUpdate*) payload;
     QXLRect *qxl_dirty_rects = msg->qxl_dirty_rects;
 
     spice_return_if_fail(red_qxl_is_running(worker->qxl));
@@ -367,31 +352,27 @@ static void handle_dev_update(void *opaque, void *payload)
     }
 }
 
-static void handle_dev_del_memslot(void *opaque, void *payload)
+static void
+handle_dev_del_memslot(RedWorker* worker, RedWorkerMessageDelMemslot* msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageDelMemslot*) payload;
     uint32_t slot_id = msg->slot_id;
     uint32_t slot_group_id = msg->slot_group_id;
 
     memslot_info_del_slot(&worker->mem_slots, slot_group_id, slot_id);
 }
 
-static void handle_dev_destroy_surface_wait(void *opaque, void *payload)
+static void
+handle_dev_destroy_surface_wait(RedWorker* worker, RedWorkerMessageDestroySurfaceWait* msg)
 {
-    auto msg = (RedWorkerMessageDestroySurfaceWait*) payload;
-    auto worker = (RedWorker*) opaque;
-
     spice_return_if_fail(msg->surface_id == 0);
 
     flush_all_qxl_commands(worker);
     display_channel_destroy_surface_wait(worker->display_channel, msg->surface_id);
 }
 
-static void handle_dev_destroy_surfaces(void *opaque, void *payload)
+static void
+handle_dev_destroy_surfaces(RedWorker* worker, RedWorkerMessageDestroySurfaces*)
 {
-    auto worker = (RedWorker*) opaque;
-
     flush_all_qxl_commands(worker);
     display_channel_destroy_surfaces(worker->display_channel);
     worker->cursor_channel->reset();
@@ -416,9 +397,8 @@ static void dev_create_primary_surface(RedWorker *worker, uint32_t surface_id,
         return;
     }
 
-    line_0 = (uint8_t*)memslot_get_virt(&worker->mem_slots, surface.mem,
-                                        surface.height * abs(surface.stride),
-                                        surface.group_id);
+    line_0 = static_cast<uint8_t *>(memslot_get_virt(
+        &worker->mem_slots, surface.mem, surface.height * abs(surface.stride), surface.group_id));
     if (line_0 == nullptr) {
         return;
     }
@@ -428,7 +408,7 @@ static void dev_create_primary_surface(RedWorker *worker, uint32_t surface_id,
     }
 
     if (surface.stride < 0) {
-        line_0 -= (int32_t)(surface.stride * (surface.height -1));
+        line_0 -= static_cast<int32_t>(surface.stride * (surface.height - 1));
     }
 
     display_channel_create_surface(display, 0, surface.width, surface.height, surface.stride, surface.format,
@@ -450,11 +430,9 @@ static void dev_create_primary_surface(RedWorker *worker, uint32_t surface_id,
     worker->cursor_channel->do_init();
 }
 
-static void handle_dev_create_primary_surface(void *opaque, void *payload)
+static void
+handle_dev_create_primary_surface(RedWorker* worker, RedWorkerMessageCreatePrimarySurface* msg)
 {
-    auto msg = (RedWorkerMessageCreatePrimarySurface*) payload;
-    auto worker = (RedWorker*) opaque;
-
     dev_create_primary_surface(worker, msg->surface_id, msg->surface);
 }
 
@@ -470,29 +448,23 @@ static void destroy_primary_surface(RedWorker *worker, uint32_t surface_id)
 
     flush_all_qxl_commands(worker);
     display_channel_destroy_surface_wait(display, 0);
-    display_channel_surface_unref(display, 0);
-
-    /* FIXME: accessing private data only for warning purposes...
-    spice_warn_if_fail(ring_is_empty(&display->streams));
-    */
-    spice_warn_if_fail(!display_channel_surface_has_canvas(display, surface_id));
+    display_channel_surface_id_unref(display, 0);
 
     worker->cursor_channel->reset();
 }
 
-static void handle_dev_destroy_primary_surface(void *opaque, void *payload)
+static void
+handle_dev_destroy_primary_surface(RedWorker* worker, RedWorkerMessageDestroyPrimarySurface* msg)
 {
-    auto msg = (RedWorkerMessageDestroyPrimarySurface*) payload;
-    auto worker = (RedWorker*) opaque;
     uint32_t surface_id = msg->surface_id;
 
     destroy_primary_surface(worker, surface_id);
 }
 
-static void handle_dev_destroy_primary_surface_async(void *opaque, void *payload)
+static void
+handle_dev_destroy_primary_surface_async(RedWorker* worker,
+                                         RedWorkerMessageDestroyPrimarySurfaceAsync* msg)
 {
-    auto msg = (RedWorkerMessageDestroyPrimarySurfaceAsync*) payload;
-    auto worker = (RedWorker*) opaque;
     uint32_t surface_id = msg->surface_id;
 
     destroy_primary_surface(worker, surface_id);
@@ -500,20 +472,17 @@ static void handle_dev_destroy_primary_surface_async(void *opaque, void *payload
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_flush_surfaces_async(void *opaque, void *payload)
+static void
+handle_dev_flush_surfaces_async(RedWorker* worker, RedWorkerMessageFlushSurfacesAsync* msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageFlushSurfacesAsync*) payload;
-
     flush_all_qxl_commands(worker);
     display_channel_flush_all_surfaces(worker->display_channel);
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_stop(void *opaque, void *payload)
+static void
+handle_dev_stop(RedWorker* worker, RedWorkerMessageStop*)
 {
-    auto worker = (RedWorker*) opaque;
-
     spice_debug("stop");
     if (!red_qxl_is_running(worker->qxl)) {
 	    return;
@@ -534,10 +503,9 @@ static void handle_dev_stop(void *opaque, void *payload)
     worker->cursor_channel->wait_all_sent(COMMON_CLIENT_TIMEOUT);
 }
 
-static void handle_dev_start(void *opaque, void *payload)
+static void
+handle_dev_start(RedWorker* worker, RedWorkerMessageStart*)
 {
-    auto worker = (RedWorker*) opaque;
-
     if (red_qxl_is_running(worker->qxl)) {
         return;
     }
@@ -554,17 +522,16 @@ static void handle_dev_start(void *opaque, void *payload)
     worker->event_timeout = 0;
 }
 
-static void handle_dev_wakeup(void *opaque, void *payload)
+static void
+handle_dev_wakeup(RedWorker* worker, RedWorkerMessageWakeup*)
 {
-    auto worker = (RedWorker*) opaque;
-
     stat_inc_counter(worker->wakeup_counter, 1);
     red_qxl_clear_pending(worker->qxl->st, RED_DISPATCHER_PENDING_WAKEUP);
 }
 
-static void handle_dev_oom(void *opaque, void *payload)
+static void
+handle_dev_oom(RedWorker* worker, RedWorkerMessageOom*)
 {
-    auto worker = (RedWorker*) opaque;
     DisplayChannel *display = worker->display_channel;
 
     int ring_is_empty;
@@ -583,45 +550,39 @@ static void handle_dev_oom(void *opaque, void *payload)
     red_qxl_clear_pending(worker->qxl->st, RED_DISPATCHER_PENDING_OOM);
 }
 
-static void handle_dev_reset_cursor(void *opaque, void *payload)
+static void
+handle_dev_reset_cursor(RedWorker* worker, RedWorkerMessageResetCursor*)
 {
-    auto worker = (RedWorker*) opaque;
-
     worker->cursor_channel->reset();
 }
 
-static void handle_dev_reset_image_cache(void *opaque, void *payload)
+static void
+handle_dev_reset_image_cache(RedWorker* worker, RedWorkerMessageResetImageCache*)
 {
-    auto worker = (RedWorker*) opaque;
-
     display_channel_reset_image_cache(worker->display_channel);
 }
 
-static void handle_dev_destroy_surface_wait_async(void *opaque, void *payload)
+static void
+handle_dev_destroy_surface_wait_async(RedWorker* worker,
+                                      RedWorkerMessageDestroySurfaceWaitAsync *msg)
 {
-    auto msg = (RedWorkerMessageDestroySurfaceWaitAsync*) payload;
-    auto worker = (RedWorker*) opaque;
-
     display_channel_destroy_surface_wait(worker->display_channel, msg->surface_id);
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_destroy_surfaces_async(void *opaque, void *payload)
+static void
+handle_dev_destroy_surfaces_async(RedWorker* worker, RedWorkerMessageDestroySurfacesAsync* msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageDestroySurfacesAsync*) payload;
-
     flush_all_qxl_commands(worker);
     display_channel_destroy_surfaces(worker->display_channel);
     worker->cursor_channel->reset();
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_create_primary_surface_async(void *opaque, void *payload)
+static void
+handle_dev_create_primary_surface_async(RedWorker* worker,
+                                        RedWorkerMessageCreatePrimarySurfaceAsync* msg)
 {
-    auto msg = (RedWorkerMessageCreatePrimarySurfaceAsync*) payload;
-    auto worker = (RedWorker*) opaque;
-
     dev_create_primary_surface(worker, msg->surface_id, msg->surface);
     red_qxl_create_primary_surface_complete(worker->qxl->st, &msg->surface);
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
@@ -632,15 +593,13 @@ static inline uint32_t qxl_monitors_config_size(uint32_t heads)
     return sizeof(QXLMonitorsConfig) + sizeof(QXLHead) * heads;
 }
 
-static void handle_dev_monitors_config_async(void *opaque, void *payload)
+static void
+handle_dev_monitors_config_async(RedWorker* worker, RedWorkerMessageMonitorsConfigAsync* msg)
 {
-    auto msg = (RedWorkerMessageMonitorsConfigAsync*) payload;
-    auto worker = (RedWorker*) opaque;
     uint16_t count, max_allowed;
     const QXLMonitorsConfig *dev_monitors_config =
-        (QXLMonitorsConfig*)memslot_get_virt(&worker->mem_slots, msg->monitors_config,
-                                             qxl_monitors_config_size(1),
-                                             msg->group_id);
+        static_cast<QXLMonitorsConfig *>(memslot_get_virt(
+            &worker->mem_slots, msg->monitors_config, qxl_monitors_config_size(1), msg->group_id));
 
     if (dev_monitors_config == nullptr) {
         /* TODO: raise guest bug (requires added QXL interface) */
@@ -661,10 +620,8 @@ static void handle_dev_monitors_config_async(void *opaque, void *payload)
         goto async_complete;
     }
     /* get pointer again to check virtual size */
-    dev_monitors_config =
-        (QXLMonitorsConfig*)memslot_get_virt(&worker->mem_slots, msg->monitors_config,
-                                             qxl_monitors_config_size(count),
-                                             msg->group_id);
+    dev_monitors_config = static_cast<QXLMonitorsConfig *>(memslot_get_virt(
+        &worker->mem_slots, msg->monitors_config, qxl_monitors_config_size(count), msg->group_id));
     if (dev_monitors_config == nullptr) {
         /* TODO: raise guest bug (requires added QXL interface) */
         goto async_complete;
@@ -676,10 +633,9 @@ async_complete:
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_set_compression(void *opaque, void *payload)
+static void
+handle_dev_set_compression(RedWorker* worker, RedWorkerMessageSetCompression* msg)
 {
-    auto msg = (RedWorkerMessageSetCompression*) payload;
-    auto worker = (RedWorker*) opaque;
     SpiceImageCompression image_compression = msg->image_compression;
 
     display_channel_set_image_compression(worker->display_channel, image_compression);
@@ -688,28 +644,22 @@ static void handle_dev_set_compression(void *opaque, void *payload)
     display_channel_compress_stats_reset(worker->display_channel);
 }
 
-static void handle_dev_set_streaming_video(void *opaque, void *payload)
+static void
+handle_dev_set_streaming_video(RedWorker* worker, RedWorkerMessageSetStreamingVideo* msg)
 {
-    auto msg = (RedWorkerMessageSetStreamingVideo*) payload;
-    auto worker = (RedWorker*) opaque;
-
     display_channel_set_stream_video(worker->display_channel, msg->streaming_video);
 }
 
-static void handle_dev_set_video_codecs(void *opaque, void *payload)
+static void
+handle_dev_set_video_codecs(RedWorker* worker, RedWorkerMessageSetVideoCodecs* msg)
 {
-    auto msg = (RedWorkerMessageSetVideoCodecs*) payload;
-    auto worker = (RedWorker*) opaque;
-
     display_channel_set_video_codecs(worker->display_channel, msg->video_codecs);
     g_array_unref(msg->video_codecs);
 }
 
-static void handle_dev_set_mouse_mode(void *opaque, void *payload)
+static void
+handle_dev_set_mouse_mode(RedWorker* worker, RedWorkerMessageSetMouseMode* msg)
 {
-    auto msg = (RedWorkerMessageSetMouseMode*) payload;
-    auto worker = (RedWorker*) opaque;
-
     spice_debug("mouse mode %u", msg->mode);
     worker->cursor_channel->set_mouse_mode(msg->mode);
 }
@@ -721,10 +671,9 @@ static void dev_add_memslot(RedWorker *worker, QXLDevMemSlot mem_slot)
                           mem_slot.generation);
 }
 
-static void handle_dev_add_memslot(void *opaque, void *payload)
+static void
+handle_dev_add_memslot(RedWorker* worker, RedWorkerMessageAddMemslot* msg)
 {
-    auto worker = (RedWorker*) opaque;
-    auto msg = (RedWorkerMessageAddMemslot*) payload;
     QXLDevMemSlot mem_slot = msg->mem_slot;
 
     memslot_info_add_slot(&worker->mem_slots, mem_slot.slot_group_id, mem_slot.slot_id,
@@ -732,49 +681,40 @@ static void handle_dev_add_memslot(void *opaque, void *payload)
                           mem_slot.generation);
 }
 
-static void handle_dev_add_memslot_async(void *opaque, void *payload)
+static void
+handle_dev_add_memslot_async(RedWorker* worker, RedWorkerMessageAddMemslotAsync* msg)
 {
-    auto msg = (RedWorkerMessageAddMemslotAsync*) payload;
-    auto worker = (RedWorker*) opaque;
-
     dev_add_memslot(worker, msg->mem_slot);
     red_qxl_async_complete(worker->qxl, msg->base.cookie);
 }
 
-static void handle_dev_reset_memslots(void *opaque, void *payload)
+static void
+handle_dev_reset_memslots(RedWorker* worker, RedWorkerMessageResetMemslots*)
 {
-    auto worker = (RedWorker*) opaque;
-
     memslot_info_reset(&worker->mem_slots);
 }
 
-static void handle_dev_driver_unload(void *opaque, void *payload)
+static void
+handle_dev_driver_unload(RedWorker* worker, RedWorkerMessageDriverUnload*)
 {
-    auto worker = (RedWorker*) opaque;
-
     worker->driver_cap_monitors_config = false;
 }
 
-static
-void handle_dev_gl_scanout(void *opaque, void *payload)
+static void
+handle_dev_gl_scanout(RedWorker* worker, RedWorkerMessageGlScanout*)
 {
-    auto worker = (RedWorker*) opaque;
-
     display_channel_gl_scanout(worker->display_channel);
 }
 
-static
-void handle_dev_gl_draw_async(void *opaque, void *payload)
+static void
+handle_dev_gl_draw_async(RedWorker* worker, RedWorkerMessageGlDraw* draw)
 {
-    auto worker = (RedWorker*) opaque;
-    auto draw = (RedWorkerMessageGlDraw*) payload;
-
     display_channel_gl_draw(worker->display_channel, &draw->draw);
 }
 
-static void handle_dev_close(void *opaque, void *payload)
+static void
+handle_dev_close(RedWorker* worker, RedWorkerMessageClose*)
 {
-    auto worker = (RedWorker*) opaque;
     g_main_loop_quit(worker->loop);
 }
 
@@ -794,10 +734,9 @@ static bool loadvm_command(RedWorker *worker, QXLCommandExt *ext)
     return TRUE;
 }
 
-static void handle_dev_loadvm_commands(void *opaque, void *payload)
+static void
+handle_dev_loadvm_commands(RedWorker* worker, RedWorkerMessageLoadvmCommands* msg)
 {
-    auto msg = (RedWorkerMessageLoadvmCommands*) payload;
-    auto worker = (RedWorker*) opaque;
     uint32_t i;
     uint32_t count = msg->count;
     QXLCommandExt *ext = msg->ext;
@@ -813,138 +752,118 @@ static void handle_dev_loadvm_commands(void *opaque, void *payload)
 
 static void worker_dispatcher_record(void *opaque, uint32_t message_type, void *payload)
 {
-    auto worker = (RedWorker*) opaque;
+    auto worker = static_cast<RedWorker *>(opaque);
 
     red_record_event(worker->record, 1, message_type);
 }
 
+template <typename T>
+static inline void
+register_handler(Dispatcher *dispatcher,
+                 void (*handler)(RedWorker*, T*), bool ack)
+{
+    dispatcher->register_handler(T::MESSAGE_NUM,
+                                 reinterpret_cast<dispatcher_handle_message>(handler),
+                                 sizeof(T),
+                                 ack);
+}
+
 static void register_callbacks(Dispatcher *dispatcher)
 {
     /* TODO: register cursor & display specific msg in respective channel files */
-    dispatcher->register_handler(RED_WORKER_MESSAGE_UPDATE,
-                                 handle_dev_update,
-                                 sizeof(RedWorkerMessageUpdate),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_UPDATE_ASYNC,
-                                 handle_dev_update_async,
-                                 sizeof(RedWorkerMessageUpdateAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_ADD_MEMSLOT,
-                                 handle_dev_add_memslot,
-                                 sizeof(RedWorkerMessageAddMemslot),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_ADD_MEMSLOT_ASYNC,
-                                 handle_dev_add_memslot_async,
-                                 sizeof(RedWorkerMessageAddMemslotAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DEL_MEMSLOT,
-                                 handle_dev_del_memslot,
-                                 sizeof(RedWorkerMessageDelMemslot),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_SURFACES,
-                                 handle_dev_destroy_surfaces,
-                                 sizeof(RedWorkerMessageDestroySurfaces),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_SURFACES_ASYNC,
-                                 handle_dev_destroy_surfaces_async,
-                                 sizeof(RedWorkerMessageDestroySurfacesAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_PRIMARY_SURFACE,
-                                 handle_dev_destroy_primary_surface,
-                                 sizeof(RedWorkerMessageDestroyPrimarySurface),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_PRIMARY_SURFACE_ASYNC,
-                                 handle_dev_destroy_primary_surface_async,
-                                 sizeof(RedWorkerMessageDestroyPrimarySurfaceAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_CREATE_PRIMARY_SURFACE_ASYNC,
-                                 handle_dev_create_primary_surface_async,
-                                 sizeof(RedWorkerMessageCreatePrimarySurfaceAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_CREATE_PRIMARY_SURFACE,
-                                 handle_dev_create_primary_surface,
-                                 sizeof(RedWorkerMessageCreatePrimarySurface),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_RESET_IMAGE_CACHE,
-                                 handle_dev_reset_image_cache,
-                                 sizeof(RedWorkerMessageResetImageCache),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_RESET_CURSOR,
-                                 handle_dev_reset_cursor,
-                                 sizeof(RedWorkerMessageResetCursor),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_WAKEUP,
-                                 handle_dev_wakeup,
-                                 sizeof(RedWorkerMessageWakeup),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_OOM,
-                                 handle_dev_oom,
-                                 sizeof(RedWorkerMessageOom),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_START,
-                                 handle_dev_start,
-                                 sizeof(RedWorkerMessageStart),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_FLUSH_SURFACES_ASYNC,
-                                 handle_dev_flush_surfaces_async,
-                                 sizeof(RedWorkerMessageFlushSurfacesAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_STOP,
-                                 handle_dev_stop,
-                                 sizeof(RedWorkerMessageStop),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_LOADVM_COMMANDS,
-                                 handle_dev_loadvm_commands,
-                                 sizeof(RedWorkerMessageLoadvmCommands),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_SET_COMPRESSION,
-                                 handle_dev_set_compression,
-                                 sizeof(RedWorkerMessageSetCompression),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_SET_STREAMING_VIDEO,
-                                 handle_dev_set_streaming_video,
-                                 sizeof(RedWorkerMessageSetStreamingVideo),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_SET_VIDEO_CODECS,
-                                 handle_dev_set_video_codecs,
-                                 sizeof(RedWorkerMessageSetVideoCodecs),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_SET_MOUSE_MODE,
-                                 handle_dev_set_mouse_mode,
-                                 sizeof(RedWorkerMessageSetMouseMode),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_SURFACE_WAIT,
-                                 handle_dev_destroy_surface_wait,
-                                 sizeof(RedWorkerMessageDestroySurfaceWait),
-                                 true);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DESTROY_SURFACE_WAIT_ASYNC,
-                                 handle_dev_destroy_surface_wait_async,
-                                 sizeof(RedWorkerMessageDestroySurfaceWaitAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_RESET_MEMSLOTS,
-                                 handle_dev_reset_memslots,
-                                 sizeof(RedWorkerMessageResetMemslots),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_MONITORS_CONFIG_ASYNC,
-                                 handle_dev_monitors_config_async,
-                                 sizeof(RedWorkerMessageMonitorsConfigAsync),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_DRIVER_UNLOAD,
-                                 handle_dev_driver_unload,
-                                 sizeof(RedWorkerMessageDriverUnload),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_GL_SCANOUT,
-                                 handle_dev_gl_scanout,
-                                 sizeof(RedWorkerMessageGlScanout),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_GL_DRAW_ASYNC,
-                                 handle_dev_gl_draw_async,
-                                 sizeof(RedWorkerMessageGlDraw),
-                                 false);
-    dispatcher->register_handler(RED_WORKER_MESSAGE_CLOSE_WORKER,
-                                 handle_dev_close,
-                                 sizeof(RedWorkerMessageClose),
-                                 false);
+    register_handler(dispatcher,
+                     handle_dev_update,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_update_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_add_memslot,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_add_memslot_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_del_memslot,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_destroy_surfaces,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_destroy_surfaces_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_destroy_primary_surface,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_destroy_primary_surface_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_create_primary_surface_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_create_primary_surface,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_reset_image_cache,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_reset_cursor,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_wakeup,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_oom,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_start,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_flush_surfaces_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_stop,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_loadvm_commands,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_set_compression,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_set_streaming_video,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_set_video_codecs,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_set_mouse_mode,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_destroy_surface_wait,
+                     true);
+    register_handler(dispatcher,
+                     handle_dev_destroy_surface_wait_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_reset_memslots,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_monitors_config_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_driver_unload,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_gl_scanout,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_gl_draw_async,
+                     false);
+    register_handler(dispatcher,
+                     handle_dev_close,
+                     false);
 }
 
 
@@ -1085,7 +1004,7 @@ RedWorker* red_worker_new(QXLInstance *qxl)
 
 static void *red_worker_main(void *arg)
 {
-    auto worker = (RedWorker *) arg;
+    auto worker = static_cast<RedWorker *>(arg);
 
     spice_debug("begin");
 #if defined(__APPLE__)
