@@ -18,9 +18,9 @@ def get_named_types():
 # only to attributes that affect pointer or array attributes, as these
 # are member local types, unlike e.g. a Struct that may be used by
 # other members
-propagated_attributes=["ptr_array", "nonnull", "chunk"]
+propagated_attributes=["ptr_array", "nonnull", "chunk", "zero_terminated"]
 
-valid_attributes=set([
+valid_attributes_generic=set([
     # embedded/appended at the end of the resulting C structure
     'end',
     # the C structure contains a pointer to data
@@ -86,7 +86,17 @@ attributes_with_arguments=set([
     'virtual',
 ])
 
-def fix_attributes(attribute_list):
+# these attributes specify output format, only one can be set
+output_attributes = set([
+    'end',
+    'to_ptr',
+    'as_ptr',
+    'ptr_array',
+    'zero',
+    'chunk',
+])
+
+def fix_attributes(attribute_list, valid_attributes=valid_attributes_generic):
     attrs = {}
     for attr in attribute_list:
         name = attr[0][1:] # [1:] strips the leading '@' from the name
@@ -100,9 +110,8 @@ def fix_attributes(attribute_list):
             raise Exception("Attribute %s has more than 1 argument" % name)
         attrs[name] = lst
 
-    # these attributes specify output format, only one can be set
-    output_attrs = set(['end', 'to_ptr', 'as_ptr', 'ptr_array', 'zero', 'chunk'])
-    if len(output_attrs.intersection(attrs.keys())) > 1:
+    # only one output attribute can be set
+    if len(output_attributes.intersection(attrs.keys())) > 1:
         raise Exception("Multiple output type attributes specified %s" % output_attrs)
 
     return attrs
@@ -173,8 +182,6 @@ class Type:
         _types_by_name[self.name] = self
 
     def has_attr(self, name):
-        if not name in valid_attributes:
-            raise Exception('attribute %s not expected' % name)
         return name in self.attributes
 
 class TypeRef(Type):
@@ -310,9 +317,10 @@ class EnumType(EnumBaseType):
         last = -1
         names = {}
         values = {}
+        attributes = {}
         for v in enums:
             name = v[0]
-            if len(v) > 1:
+            if v[1] is not None:
                 value = v[1]
             else:
                 value = last + 1
@@ -321,9 +329,11 @@ class EnumType(EnumBaseType):
             assert value not in names
             names[value] = name
             values[name] = value
+            attributes[value] = fix_attributes(v[2], set(['deprecated']))
 
         self.names = names
         self.values = values
+        self.val_attributes = attributes
 
         self.attributes = fix_attributes(attribute_list)
 
@@ -341,6 +351,8 @@ class EnumType(EnumBaseType):
             writer.write(self.c_enumname(i))
             if i != current_default:
                 writer.write(" = %d" % (i))
+            if 'deprecated' in self.val_attributes[i]:
+                writer.write(' SPICE_GNUC_DEPRECATED')
             writer.write(",")
             writer.newline()
             current_default = i + 1
@@ -363,9 +375,10 @@ class FlagsType(EnumBaseType):
         last = -1
         names = {}
         values = {}
+        attributes = {}
         for v in flags:
             name = v[0]
-            if len(v) > 1:
+            if v[1] is not None:
                 value = v[1]
             else:
                 value = last + 1
@@ -374,9 +387,11 @@ class FlagsType(EnumBaseType):
             assert value not in names
             names[value] = name
             values[name] = value
+            attributes[value] = fix_attributes(v[2], set(['deprecated']))
 
         self.names = names
         self.values = values
+        self.val_attributes = attributes
 
         self.attributes = fix_attributes(attribute_list)
 
@@ -394,6 +409,8 @@ class FlagsType(EnumBaseType):
             writer.write(self.c_enumname(i))
             mask = mask |  (1<<i)
             writer.write(" = (1 << %d)" % (i))
+            if 'deprecated' in self.val_attributes[i]:
+                writer.write(' SPICE_GNUC_DEPRECATED')
             writer.write(",")
             writer.newline()
             current_default = i + 1
@@ -440,11 +457,6 @@ class ArrayType(Type):
             return False
         return self.size[0] == "image_size"
 
-    def is_bytes_length(self):
-        if isinstance(self.size, int) or isinstance(self.size, str):
-            return False
-        return self.size[0] == "bytes"
-
     def is_cstring_length(self):
         if isinstance(self.size, int) or isinstance(self.size, str):
             return False
@@ -489,6 +501,16 @@ class ArrayType(Type):
         return self.element_type.c_type()
 
     def check_valid(self, member):
+        # If the size is not constant the array has to be allocated in some
+        # way in the output and so there must be a specification for the
+        # output (as default is write into the C structure all data).
+        # The only exceptions are when the length is constant (in this case
+        # a constant length array in the C structure is used) or a pointer
+        # (in this case the pointer allocate the array).
+        if (not self.is_constant_length()
+            and len(output_attributes.intersection(member.attributes.keys())) == 0
+            and not member.member_type.is_pointer()):
+            raise Exception("Array length must be a constant or some output specifiers must be set")
         # These attribute corresponds to specific structure size
         if member.has_attr("chunk") or member.has_attr("as_ptr"):
             return
@@ -531,6 +553,8 @@ class ArrayType(Type):
             return writer.writeln('%s *%s[0];' % (self.c_type(), name))
         if member.has_end_attr():
             return writer.writeln('%s %s[0];' % (self.c_type(), name))
+        if self.is_constant_length() and self.has_attr("zero_terminated"):
+            return writer.writeln('%s %s[%s];' % (self.c_type(), name, self.size + 1))
         if self.is_constant_length():
             return writer.writeln('%s %s[%s];' % (self.c_type(), name, self.size))
         if self.is_identifier_length():
@@ -600,8 +624,6 @@ class Containee:
         return not self.is_switch() and self.member_type.is_primitive()
 
     def has_attr(self, name):
-        if not name in valid_attributes:
-            raise Exception('attribute %s not expected' % name)
         return name in self.attributes
 
     def has_end_attr(self):
@@ -621,6 +643,8 @@ class Member(Containee):
         for i in propagated_attributes:
             if self.has_attr(i):
                 self.member_type.attributes[i] = self.attributes[i]
+                if self.member_type.is_pointer():
+                    self.member_type.target_type.attributes[i] = self.attributes[i]
         return self
 
     def contains_member(self, member):
